# ../addons/eventscripts/playtime/playtime.py


# Time formatting:

# 1 = 1 hour 33 minutes 07 seconds
# 2 = 1 h 33 m 07 s
# 3 = 1:33:07

time_format = 1


import es
import os
import time
import cPickle
import cmdlib
import popuplib
from operator import itemgetter

playtime = {}
timer = {}

info = es.AddonInfo()
info.author = "h!gh voltage"
info.name = "Playtime Tracker"
info.basename = "playtime"
info.version = "1.2"
es.ServerVar('playtime_tracker', info.version).makepublic()

playtimepath = os.path.join( es.getAddonPath( info.basename ), "playtime.db" )
if os.path.isfile(playtimepath):
    file = open(playtimepath, 'r')
    playtime   = cPickle.load(file)
    file.close()

def load():
    cmdlib.registerSayCommand('!playtime', playtimecommand, 'Playtime Command')
    for uid in es.getUseridList():
        timer[uid] = time.time()

def savedb():
    file = open(playtimepath, 'w')
    cPickle.dump(playtime, file)
    file.close()

def unload():
    cmdlib.unregisterSayCommand('!playtime')
    savedb()

def es_map_start(ev):
    savedb()

def round_end(ev):
    savedb()

def player_activate(ev):
    userid = int(ev['userid'])
    steamid = es.getplayersteamid(userid)
    timer[userid] = time.time()
    lastconnect = time.strftime("%a, %d %b %Y %H:%M:%S"))
    if steamid != 'BOT':
        if steamid not in playtime:
            playtime[steamid] = {}
            playtime[steamid]['name'] = es.getplayername(userid)
            playtime[steamid]['total'] = 0
            playtime[steamid]['last'] = 0
        playtime[steamid]['name'] = es.getplayername(userid)
        playtime[steamid]['last'] = lastconnect

def player_disconnect(ev):
    userid = int(ev['userid'])
    steamid = ev['networkid']
    lastconnect = time.strftime("%a, %d %b %Y %H:%M:%S"))
    if steamid != 'BOT':
        if steamid not in playtime:
            playtime[steamid] = {}
            playtime[steamid]['name'] = ev['name']
            playtime[steamid]['total'] = 0
            playtime[steamid]['last'] = 0
        playtime[steamid]['last'] = lastconnect
    if userid in timer:
        elapsed = time.time() - timer[userid]
        playtime[steamid]['total'] += elapsed
        del timer[userid]

def format(s):
    hours, minutes = divmod(s, 3600)
    minutes, s = divmod(minutes, 60)
    if time_format == 1:
        return "%s hours %02i minutes %02i seconds" % (int(hours), minutes, s)
    elif time_format == 2:
        return "%s h %02i m %02i s" % (int(hours), minutes, s)
    elif time_format == 3:
        return "%s:%02i:%02i" % (int(hours), minutes, s)

def playtimecommand(userid, args):
    if not args:
        menu = popuplib.easymenu('ptmenu', None, None)
        menu.settitle("Top Playtime Players")
        sortedlist = sortedplaytime()
        if sortedlist:
            for player in sortedlist:
                menu.addoption(None, "%s: %s\nLast Connect: %s" %(playtime[player[0]]['name'], format(playtime[player[0]]['total']), playtime[player[0]]['last']))
        else:
            menu.addoption(None, "No playtimes recorded.")
        menu.unsend(userid)
        menu.send(userid)
        steamid = es.getplayersteamid(userid)
        es.tell(userid, '#multi', '#greenYou have been played for: #lightgreen%s #greenon this server.'%format(playtime[steamid]['total']))
    else:
        target = es.getuserid(args)
        if not target:
            es.tell(userid, '#multi', '#greenSorry, couldn\'t find #lightgreen%s #greenon the server.'%args)
            return
        steamid = es.getplayersteamid(target)
        name = playtime[steamid]['name']
        es.tell(userid, '#multi', '#lightgreen%s #greenhas been played for #lightgreen%s #greenon this server.'%(name, format(playtime[steamid]['total'])))        

def sortedplaytime():
    psort = {}
    for sid in playtime:
        psort[sid] = playtime[sid]["total"]
    return sorted(psort.items(), key=itemgetter(1), reverse=True)